@@ -1,23 +1,33 @@
 import requests
 import json
 import webbrowser
+import urllib
 
 
 class Visualization(object):
 
-    def __init__(self, session=None, json=None):
+    def __init__(self, session=None, json=None, auth=None):
         self.session = session
         self.id = json.get('id')
-
+        self.auth = auth
 
     def _format_url(self, url):
-        return url + '?host=' + self.session.host
+        if not url.endswith('/'):
+            url += '/'
+
+        return url + '?host=' + urllib.quote(self.session.host)
+
+    def update_image(self, image):
+        url = self.session.host + '/sessions/' + str(self.session.id) + '/visualizations/' + str(self.id) + '/data/images'
+        url = self._format_url(url)
+        files = {'file': image}
+        return requests.put(url, files=files, data={'type': 'image'}, auth=self.auth)
 
     def append_image(self, image):
         url = self.session.host + '/sessions/' + str(self.session.id) + '/visualizations/' + str(self.id) + '/data/images'
         url = self._format_url(url)
         files = {'file': image}
-        return requests.post(url, files=files, data={'type': 'image'})
+        return requests.post(url, files=files, data={'type': 'image'}, auth=self.auth)
 
     def append_data(self, data=None, field=None):
         payload = {'data': data}
@@ -27,7 +37,7 @@ class Visualization(object):
             url += field
 
         url = self._format_url(url)
-        return requests.post(url, data=json.dumps(payload), headers=headers)
+        return requests.post(url, data=json.dumps(payload), headers=headers, auth=self.auth)
 
     def update_data(self, data=None, field=None):
         payload = {'data': data}
@@ -37,7 +47,7 @@ class Visualization(object):
             url += field
 
         url = self._format_url(url)
-        return requests.put(url, data=json.dumps(payload), headers=headers)
+        return requests.put(url, data=json.dumps(payload), headers=headers, auth=self.auth)
 
     def get_permalink(self):
         return self.session.host + '/visualizations/' + str(self.id)
@@ -46,15 +56,18 @@ class Visualization(object):
         return self._format_url(self.get_permalink() + '/embed')
 
     def get_html(self):
-        import urllib2
 
-        response = urllib2.urlopen(self.get_embed_link())
-
-        return response.read()
+        r = requests.get(self.get_embed_link(), auth=self.auth)
+        return r.text
 
     def open(self):
         webbrowser.open(self.session.host + '/visualizations/' + str(self.id) + '/')
 
+    def delete(self):
+        url = self.get_permalink()
+        return requests.delete(url)
+
+
     @classmethod
     def create(cls, session=None, data=None, images=None, type=None):
         url = session.host + '/sessions/' + str(session.id) + '/visualizations'
@@ -62,30 +75,25 @@ class Visualization(object):
         if not images:
             payload = {'data': data, 'type': type}
             headers = {'Content-type': 'application/json', 'Accept': 'text/plain'}
-            
-            r = requests.post(url, data=json.dumps(payload), headers=headers)
+
+            r = requests.post(url, data=json.dumps(payload), headers=headers, auth=session.auth)
 
             if not r.status_code == requests.codes.ok:
                 raise Exception('Problem uploading data')
 
-            viz = cls(session=session, json=r.json())
+            viz = cls(session=session, json=r.json(), auth=session.auth)
 
         else:
-            if not type:
-                if len(images) > 1:
-                    type = 'volume'
-                else:
-                    type = 'image'
 
             first_image, remaining_images = images[0], images[1:]
             files = {'file': first_image}
             
-            r = requests.post(url, files=files, data={'type': type})
+            r = requests.post(url, files=files, data={'type': type}, auth=session.auth)
 
             if not r.status_code == requests.codes.ok:
                 raise Exception('Problem uploading images')
 
-            viz = Visualization(session=session, json=r.json())
+            viz = cls(session=session, json=r.json(), auth=session.auth)
             for image in remaining_images:
                 viz.append_image(image)
 
